
#include <iterator>
#include <iostream>
#include <vector>
#include <cassert>
#include <cstdio>

#include <kdev-pg-replacement.h>

#include "R_ast.h"
#include "R_default_visitor.h"
#include "R_parser.h"

namespace R {

  template <class _TokenStream = kdev_pg_token_stream>
  class Tokenize {
  public:
    typedef typename _TokenStream::token_type token_type;

    inline void init_token(token_type &tk, int kind, size_t begin = 0, size_t end = 0) {
      tk.kind = kind;
      tk.begin = begin;
      tk.end = end;
    }

  public:
    Tokenize(_TokenStream *token_stream):
      token_stream(token_stream) {}

    template <typename _Iterator>
    void operator()(_Iterator first, _Iterator last) {
      _Iterator it = first;
      while (it != last) {
	if (! (isalpha(*it) || *it == '_'))
	  ++it;
	else {
	  _Iterator start = it;
	  std::string yytext;

	  do { yytext.push_back(*it); ++it; }
	  while (it != last && (isalnum(*it) || *it == '_'));

	  int kind = parser::Token_ID;

	  if (yytext == "as")
	    kind = parser::Token_AS;

	  init_token(token_stream->next(), kind, start - first, it - first);
	}
      }

      init_token(token_stream->next(), parser::Token_EOF, it - first, it - first);
    }

  private:
    _TokenStream *token_stream;
  };

} // ::R

int main() {
  std::string contents = "cool as Sonic Youth";

  R::parser::token_stream_type token_stream;
  R::Tokenize<> tokenize(&token_stream);

  tokenize(contents.begin(), contents.end());

  R::parser::memory_pool_type memory_pool;

  R::parser parser;
  parser.set_memory_pool(&memory_pool);
  parser.set_token_stream(&token_stream);
  parser.rewind(0);

  R::declaration_ast *ast = 0;
  if (! parser.parse_declaration(&ast)) {
    assert(0);
    exit(EXIT_FAILURE);
  }

  replacement_list<> r;
  r.set_contents(contents);
  r.set_token_stream(parser.token_stream);

  r.insert_text_before(ast, "(");
  r.replace_text(ast->type, "Kim Gordon");
  r.insert_text_after(ast, ")");

  std::string out;
  r(std::back_inserter(out));

  std::cout << contents << " -> " << out << std::endl;
  
  return EXIT_SUCCESS;
}

void R::parser::yy_expected_symbol(int, char const *) {
  assert(0);
}

void R::parser::yy_expected_token(int, std::size_t, char const *) {
  assert(0);
}

